import { render, fireEvent, screen } from '@testing-library/react';
import { mount } from 'enzyme';
import React from 'react';
import { Button } from '../button.js';
describe('<Button>', () => {
  let consoleSpy;
  beforeEach(() => {
    consoleSpy = jest.spyOn(console, 'debug').mockImplementation();
  });
  afterEach(() => {
    consoleSpy.mockRestore();
  });
  describe('warning for missing aria-label and title', () => {
    it('No warning  if children exist but aria-label and title is missing', () => {
      render(/*#__PURE__*/React.createElement(Button, null, "Children content"));
      expect(consoleSpy).not.toHaveBeenCalled();
    });
    it('does not warn if aria-label and title is present', () => {
      render(/*#__PURE__*/React.createElement(Button, {
        "aria-label": "Test",
        title: "Test"
      }, "Children content"));
      expect(consoleSpy).not.toHaveBeenCalled();
    });
    it('warns if no children are present with no arial-label and title', () => {
      render(/*#__PURE__*/React.createElement(Button, null));
      expect(consoleSpy).toHaveBeenCalledWith('Button component has no children but is missing title and ariaLabel attribute.');
    });
    it('No warning if there are no children but arial label and title', () => {
      render(/*#__PURE__*/React.createElement(Button, {
        "aria-label": "Test",
        title: "Test"
      }));
      expect(consoleSpy).not.toHaveBeenCalled();
    });
  });
  it('renders a default data-test attribute', () => {
    const dataTest = 'dhis2-uicore-button';
    const wrapper = mount(/*#__PURE__*/React.createElement(Button, {
      dataTest: dataTest
    }));
    const actual = wrapper.find({
      'data-test': dataTest
    });
    expect(actual.length).toBe(1);
  });
  it('renders a custom data-test attribute', () => {
    const dataTest = 'button-data-test';
    const wrapper = mount(/*#__PURE__*/React.createElement(Button, {
      dataTest: dataTest
    }));
    const actual = wrapper.find({
      'data-test': dataTest
    });
    expect(actual.length).toBe(1);
  });
  it('has the accessibility attributes', () => {
    const dataTest = 'button-data-test';
    const wrapper = mount(/*#__PURE__*/React.createElement(Button, {
      dataTest: dataTest,
      ariaLabel: "test aria label",
      title: "title for button"
    }));
    const buttonElement = wrapper.find('button').getDOMNode();
    expect(buttonElement).toHaveAttribute('title', 'title for button');
    expect(buttonElement).toHaveAttribute('ariaLabel', 'test aria label');
  });
  describe('toggle', () => {
    it('should have class "toggled" if toggled-prop is true', () => {
      const wrapper = mount(/*#__PURE__*/React.createElement(Button, {
        toggled: true
      }));
      const actual = wrapper.find('button');
      expect(actual.hasClass('toggled')).toBe(true);
    });
    it('should not have class "toggled" if toggled-prop is not passed', () => {
      const wrapper = mount(/*#__PURE__*/React.createElement(Button, null));
      const actual = wrapper.find('button');
      expect(actual.hasClass('toggled')).toBe(false);
    });
  });
  it('should call the onKeyDown callback when provided', () => {
    const onKeyDown = jest.fn();
    render(/*#__PURE__*/React.createElement(Button, {
      name: "button-name",
      value: "button-value",
      onKeyDown: onKeyDown
    }, "btn"));
    fireEvent.keyDown(screen.getByRole('button'), {
      key: 'Enter',
      code: 'Enter',
      charCode: 13
    });
    expect(onKeyDown).toHaveBeenCalledWith({
      name: 'button-name',
      value: 'button-value'
    }, expect.objectContaining({}));
    expect(onKeyDown).toHaveBeenCalledTimes(1);
  });
});